package com.sencorsta.utils.object;

import cn.hutool.core.util.ObjectUtil;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.introspect.VisibilityChecker;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import lombok.Getter;

import java.io.IOException;
import java.text.SimpleDateFormat;

/**
 * json 封装类
 *
 * @author ICe
 */
public class Jsons {
    @Getter
    public static ObjectMapper mapper = new ObjectMapper();

    static {
        mapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:SS"));
        mapper.configure(JsonParser.Feature.AUTO_CLOSE_SOURCE, true);
        mapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        mapper.disable(SerializationFeature.FAIL_ON_EMPTY_BEANS);
        mapper.setVisibility(VisibilityChecker.Std.defaultInstance().withFieldVisibility(JsonAutoDetect.Visibility.ANY));
    }

    public static <T> T toBean(byte[] data, Class<T> valueType) {
        if (ObjectUtil.isEmpty(data)) {
            return null;
        }
        try {
            return Jsons.mapper.readValue(data, valueType);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static <T> T toBean(byte[] data, TypeReference<T> valueType) {
        if (ObjectUtil.isEmpty(data)) {
            return null;
        }
        try {
            return Jsons.mapper.readValue(data, valueType);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static <T> byte[] serializeBytes(T object) {
        try {
            return Jsons.mapper.writeValueAsBytes(object);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static <T> T deserializeBytes(byte[] data, Class<T> clazz) {
        try {
            return Jsons.mapper.readValue(data, clazz);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static <T> String serialize(T object) {
        try {
            return Jsons.mapper.writeValueAsString(object);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            return null;
        }
    }

    public static <T> T deserialize(String data, Class<T> clazz) {
        try {
            return Jsons.mapper.readValue(data, clazz);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 将任意对象转为 JSON 字符串
     *
     * @param obj 待转换的对象
     * @return 转换后的 JSON 字符串
     * @throws RuntimeException 如果转换过程中出现异常
     */
    public static String toJsonString(Object obj) {
        try {
            return mapper.writeValueAsString(obj);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to convert object to JSON string.", e);
        }
    }

    /**
     * 将 JSON 字符串转为指定类型的对象
     *
     * @param jsonString 待转换的 JSON 字符串
     * @param valueType  目标对象类型
     * @param <T>        目标对象类型的泛型
     * @return 转换后的对象
     * @throws RuntimeException 如果转换过程中出现异常
     */
    public static <T> T fromJsonString(String jsonString, Class<T> valueType) {
        try {
            return mapper.readValue(jsonString, valueType);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to convert JSON string to object.", e);
        }
    }
    public static <T> T fromJsonString(String jsonString, TypeReference<T> valueTypeRef) {
        try {
            return mapper.readValue(jsonString, valueTypeRef);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to convert JSON string to object.", e);
        }
    }

    public static ObjectNode createObjectNode() {
        return mapper.createObjectNode();
    }

    public static ArrayNode createArrayNode() {
        return mapper.createArrayNode();
    }

    public static void main(String[] args) throws Exception {
        String jsonString="{\"channel\":null,\"server\":{\"type\":\"idsGateway\",\"sid\":null,\"host\":\"0.0.0.0\",\"publicHost\":\"10.198.50.16\",\"port\":10001,\"freeMemory\":385336040,\"maxMemory\":510656512}}";

        JsonNode jsonNode = mapper.readTree(jsonString);

        ObjectNode objectNode = jsonNode.deepCopy();
        System.out.println(objectNode.get("server"));
    }

}
