package com.sencorsta.ids.idsgateway.controller;


import cn.hutool.core.lang.UUID;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.sencorsta.ids.api.request.ReqUserLoginAccount;
import com.sencorsta.ids.api.request.ReqUserRegister;
import com.sencorsta.ids.api.response.ResUserLoginAccount;
import com.sencorsta.ids.api.response.ResUserRegister;
import com.sencorsta.ids.core.application.proxy.ProxyClient;
import com.sencorsta.ids.core.constant.ProtocolTypeConstant;
import com.sencorsta.ids.core.constant.SerializeTypeConstant;
import com.sencorsta.ids.core.entity.IdsRequest;
import com.sencorsta.ids.core.entity.IdsResponse;
import com.sencorsta.ids.core.entity.annotation.Autowired;
import com.sencorsta.ids.core.entity.annotation.Controller;
import com.sencorsta.ids.core.entity.annotation.RequestMapping;
import com.sencorsta.ids.core.net.protocol.RpcMessage;
import com.sencorsta.ids.idsgateway.service.IdsGatewayService;
import com.sencorsta.utils.object.Jsons;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * @author ICe
 */
@Controller
@RequestMapping("/user")
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserController {

    private final IdsGatewayService idsGatewayService;

    @RequestMapping("/loginAccount")
    public IdsResponse<ResUserLoginAccount> loginAccount(IdsRequest<ReqUserLoginAccount> request) throws Exception {
        RpcMessage message = new RpcMessage(ProtocolTypeConstant.TYPE_RPC_REQ);
        message.setUserId("HTTP_TEMP:" + UUID.fastUUID());
        message.setMethod("/user/loginAccount");
        message.setSerializeType(SerializeTypeConstant.TYPE_JSON);
        message.setData(Jsons.getMapper().writeValueAsBytes(request.getData()));
        RpcMessage res = ProxyClient.requestByTypeSync(message, "idsUser");
        ObjectNode resUserLoginAccount = Jsons.toBean(res.getData(), ObjectNode.class);
        System.out.println("resUserLoginAccount = " + resUserLoginAccount);
        IdsResponse<ResUserLoginAccount> resUserLoginAccountIdsResponse = new IdsResponse<>();
        resUserLoginAccountIdsResponse.setCode(resUserLoginAccount.get("code").intValue());
        resUserLoginAccountIdsResponse.setMessage(resUserLoginAccount.get("message").asText());
        resUserLoginAccountIdsResponse.setData(Jsons.getMapper().convertValue(resUserLoginAccount.get("data"), ResUserLoginAccount.class));
        return resUserLoginAccountIdsResponse;
    }


    @RequestMapping("/register")
    public IdsResponse<ResUserRegister> register(IdsRequest<ReqUserRegister> request) throws Exception {
        RpcMessage message = new RpcMessage(ProtocolTypeConstant.TYPE_RPC_REQ);
        message.setUserId("HTTP_TEMP:" + UUID.fastUUID());
        message.setMethod("/user/register");
        message.setSerializeType(SerializeTypeConstant.TYPE_JSON);
        message.setData(Jsons.getMapper().writeValueAsBytes(request.getData()));
        RpcMessage resSync = ProxyClient.requestByTypeSync(message, "idsUser");
        ObjectNode resUserLoginAccount = Jsons.toBean(resSync.getData(), ObjectNode.class);
        IdsResponse<ResUserRegister> response = new IdsResponse<>();
        response.setCode(resUserLoginAccount.get("code").intValue());
        response.setMessage(resUserLoginAccount.get("message").asText());
        response.setData(Jsons.getMapper().convertValue(resUserLoginAccount.get("data"), ResUserRegister.class));
        return response;
    }

}
