package com.sencorsta.idsoos.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.sencorsta.ids.ErrorCodeList;
import com.sencorsta.ids.api.request.HelloFileRequest;
import com.sencorsta.ids.api.request.ReqOosMyFiles;
import com.sencorsta.ids.api.request.ReqOosUploadFile;
import com.sencorsta.ids.api.response.ResOosMyFiles;
import com.sencorsta.ids.api.response.ResOosUploadFile;
import com.sencorsta.ids.core.constant.ErrorCodeConstant;
import com.sencorsta.ids.core.entity.ErrorCode;
import com.sencorsta.ids.core.entity.HttpFileResponse;
import com.sencorsta.ids.core.entity.IdsRequest;
import com.sencorsta.ids.core.entity.IdsResponse;
import com.sencorsta.ids.core.entity.annotation.Autowired;
import com.sencorsta.ids.core.entity.annotation.Controller;
import com.sencorsta.ids.core.entity.annotation.RequestMapping;
import com.sencorsta.ids.core.net.handle.ContentTypes;
import com.sencorsta.idsoos.service.IdsOosService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.util.Date;

import static cn.hutool.core.date.DatePattern.CHINESE_DATE_TIME_FORMATTER;

/**
 * @author ICe
 */
@Controller
@RequestMapping("/idsOos")
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class IdsOosController {

    private final IdsOosService idsOosService;

    @RequestMapping("/uploadFile")
    public IdsResponse<ResOosUploadFile> uploadFile(IdsRequest<ReqOosUploadFile> request) {
        try {
            return idsOosService.uploadFile(request);
        } catch (ErrorCode errorCode) {
            return new IdsResponse<>(errorCode);
        } catch (Exception e) {
            log.error("Exception:",e);
            return new IdsResponse<>(ErrorCodeConstant.SYSTEM_ERROR);
        }
    }

    @RequestMapping("/myFiles")
    public IdsResponse<ResOosMyFiles> myFiles(IdsRequest<ReqOosMyFiles> request) {
        try {
            return idsOosService.myFiles(request);
        } catch (ErrorCode errorCode) {
            return new IdsResponse<>(errorCode);
        } catch (Exception e) {
            log.error("Exception:",e);
            return new IdsResponse<>(ErrorCodeConstant.SYSTEM_ERROR);
        }
    }

    @RequestMapping("/helloFile")
    public IdsResponse<HttpFileResponse> helloFile(IdsRequest<HelloFileRequest> request) {
        String sourceDirectory = System.getProperty("user.dir") + File.separator;
        String pathname = ".." + File.separator + "testFile" + DateUtil.format(new Date(), CHINESE_DATE_TIME_FORMATTER) + ".zip";
        FileUtil.del(pathname);
        File file = new File(pathname);
        log.info(sourceDirectory);
        log.info(file.getAbsolutePath());
        File zip = ZipUtil.zip(sourceDirectory);
        zip.renameTo(file);

        HttpFileResponse httpFileResponse = new HttpFileResponse();
        httpFileResponse.setContentType(ContentTypes.APPLICATION_OCTET_STREAM);
        httpFileResponse.setFileName(file.getPath());
        httpFileResponse.setFilePath(file.getAbsolutePath());
        return new IdsResponse<>(httpFileResponse, ErrorCodeList.SUCCESS);
    }
}
