package com.sencorsta.ids.idsuser.service.impl;


import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTHeader;
import cn.hutool.jwt.JWTUtil;
import com.google.common.collect.Maps;
import com.sencorsta.ids.ErrorCodeList;
import com.sencorsta.ids.api.request.ReqUserLoginAccount;
import com.sencorsta.ids.api.request.ReqUserRegister;
import com.sencorsta.ids.api.request.ReqUserVerify;
import com.sencorsta.ids.api.response.ResUserLoginAccount;
import com.sencorsta.ids.api.response.ResUserRegister;
import com.sencorsta.ids.api.response.ResUserVerify;
import com.sencorsta.ids.bean.table.Account;
import com.sencorsta.ids.core.entity.IdsRequest;
import com.sencorsta.ids.core.entity.IdsResponse;
import com.sencorsta.ids.core.entity.annotation.Autowired;
import com.sencorsta.ids.core.entity.annotation.Service;
import com.sencorsta.ids.idsuser.dao.UserDao;
import com.sencorsta.ids.idsuser.service.UserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.nio.charset.StandardCharsets;
import java.util.Map;

/**
 * @author ICe
 */
@Service
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UserServiceImpl implements UserService {

    private final UserDao userServerDao;
    private static final byte[] KEY = "www.sencorsta.com/user/server/key".getBytes(StandardCharsets.UTF_8);

    @Override
    public IdsResponse<ResUserLoginAccount> loginAccount(IdsRequest<ReqUserLoginAccount> request) {
        ReqUserLoginAccount reqUserLoginAccount = request.getData();
        String account = reqUserLoginAccount.getAccount();
        String password = reqUserLoginAccount.getPassword();
        log.info("loginAccount account:{} password:{}", account, "********");

        Account record = userServerDao.findByAccount(account);
        if (record == null) {
            return new IdsResponse<>(ErrorCodeList.USER_NOT_EXISTS);
        }
        String inputPass = DigestUtil.md5Hex(password).toUpperCase();
        String dataPass = record.getPassword();
        if (ObjectUtil.notEqual(inputPass, dataPass)) {
            return new IdsResponse<>(ErrorCodeList.USER_PASS_NOT_MATCH);
        }
        String token = createToken(record.getUserId());

        ResUserLoginAccount response = new ResUserLoginAccount();
        response.setUserId(record.getUserId());
        response.setToken(token);
        return new IdsResponse<>(response);
    }

    @Override
    public IdsResponse<ResUserRegister> register(IdsRequest<ReqUserRegister> request) {

        ResUserRegister resUserRegister = new ResUserRegister();

        ReqUserRegister data = request.getData();
        String account = data.getAccount();
        String email = data.getEmail();
        String password = data.getPassword();
        String phone = data.getPhone();
        Account record = userServerDao.findByAccountOrEmailOrPhone(account, email, phone);
        if (record == null) {
            String userId = UUID.fastUUID().toString();
            Account accountBean = new Account();
            accountBean.setAccount(account);
            // 密码只保存摘要 不记录实际密码
            accountBean.setPassword(DigestUtil.md5Hex(password).toUpperCase());
            accountBean.setUserId(userId);
            accountBean.setEmail(email);
            accountBean.setPhone(phone);
            userServerDao.insertAccount(accountBean);

            String token = createToken(userId);

            resUserRegister.setUserId(userId);
            resUserRegister.setAccount(account);
            resUserRegister.setEmail(email);
            resUserRegister.setPhone(phone);
            resUserRegister.setToken(token);
        } else {
            return new IdsResponse<>(ErrorCodeList.USER_ALREADY_EXISTS);
        }
        return new IdsResponse<>(resUserRegister);
    }

    @Override
    public IdsResponse<ResUserVerify> verify(IdsRequest<ReqUserVerify> request) {
        ResUserVerify res = new ResUserVerify();

        ReqUserVerify data = request.getData();
        String token = data.getToken();

        //先检查签名是否合法
        boolean verify = verifyToken(token);
        if (verify) {
            JWT jwt = JWTUtil.parseToken(token);
            jwt.getHeader(JWTHeader.TYPE);
            JSONObject payloads = jwt.getPayloads();
            String userId = payloads.getStr("userId");
            Long expire = payloads.getLong("expire");
            log.debug("userId:{},expire:{}", userId, expire);
            //再检查时间是否过期
            if (System.currentTimeMillis() > expire) {
                verify = false;
            }
        }
        res.setVerify(verify);
        return new IdsResponse<>(res);
    }

    private boolean verifyToken(String token) {
        boolean verify = JWTUtil.verify(token, KEY);
        log.debug("verify:{}", verify);
        return verify;
    }

    private String createToken(String userId) {
        Map<String, Object> payload = Maps.newHashMapWithExpectedSize(4);
        payload.put("userId", userId);
        //30天过期
        payload.put("expire", System.currentTimeMillis() + 1000L * 60 * 60 * 24 * 30);
        String token = JWTUtil.createToken(payload, KEY);
        log.debug("createToken:{}", token);
        return token;
    }
}
