package com.sencorsta.starriverlake.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.ZipUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.sencorsta.ids.ErrorCodeList;
import com.sencorsta.ids.api.request.HelloFileRequest;
import com.sencorsta.ids.api.request.HelloWorldRequest;
import com.sencorsta.ids.core.entity.*;
import com.sencorsta.ids.core.entity.annotation.Autowired;
import com.sencorsta.ids.core.entity.annotation.Controller;
import com.sencorsta.ids.core.entity.annotation.RequestMapping;
import com.sencorsta.ids.core.net.handle.ContentTypes;
import com.sencorsta.ids.core.net.protocol.MessageJsonFactory;
import com.sencorsta.ids.core.net.protocol.RpcMessage;
import com.sencorsta.ids.core.processor.MessageProcessor;
import com.sencorsta.starriverlake.service.StarRiverLakeService;
import com.sencorsta.utils.object.Jsons;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.List;

import static cn.hutool.core.date.DatePattern.CHINESE_DATE_TIME_FORMATTER;


/**
 * @author ICe
 */
@Controller
@RequestMapping("/starRiverLake")
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class StarRiverLakeController {

    private final StarRiverLakeService starRiverLakeService;

    @RequestMapping("/helloWorld")
    public IdsResponse<String> helloWorld(IdsRequest<HelloWorldRequest> request) {

        RpcMessage message = MessageJsonFactory.newPushMessage();
        message.setMethod("/starRiverLake/helloWorld/push");
        message.setUserId(request.getUserId());
        message.setChannel(request.getChannel());
        message.setJsonData(new Server());
        MessageProcessor.push(message);

        return new IdsResponse<>(starRiverLakeService.helloWorld(request.getData().getName(),request.getChannel()));
    }

    @RequestMapping("/helloFile")
    public IdsResponse<HttpFileResponse> helloFile(IdsRequest<HelloFileRequest> request) {
        String sourceDirectory = System.getProperty("user.dir") + File.separator;
        String pathname = ".." + File.separator + "testFile" + DateUtil.format(new Date(), CHINESE_DATE_TIME_FORMATTER) + ".zip";
        FileUtil.del(pathname);
        File file = new File(pathname);
        log.info(sourceDirectory);
        log.info(file.getAbsolutePath());
        File zip = ZipUtil.zip(sourceDirectory);
        zip.renameTo(file);

        HttpFileResponse httpFileResponse = new HttpFileResponse();
        httpFileResponse.setContentType(ContentTypes.APPLICATION_OCTET_STREAM);
        httpFileResponse.setFileName(file.getPath());
        httpFileResponse.setFilePath(file.getAbsolutePath());
        return new IdsResponse<>(httpFileResponse, ErrorCodeList.SUCCESS);
    }

    @RequestMapping("/helloUpLoad")
    public IdsResponse<ObjectNode> helloUpLoad(IdsRequest<HelloFileRequest> request) {
        HelloFileRequest data = request.getData();
        List<FileUpload> files = data.getFiles();

        // 用于存储每个文件的 MD5 码和文件名
        ObjectNode result = Jsons.createObjectNode();
        try {
            for (FileUpload file : files) {
                String fileName = file.getName();
                byte[] fileData = file.getData();

                // 计算文件的 MD5 码
                String md5 = DigestUtil.md5Hex(fileData);
                // 保存文件到本地
                saveFileToLocal(md5, fileData);
                // 将文件名和对应的 MD5 码加入结果中
                result.put(fileName, md5);
            }
            // 返回结果
            return new IdsResponse<>(result);
        } catch (Exception e) {
            e.printStackTrace();
            // 处理异常情况
            return new IdsResponse<>(null, new ErrorCode(500, "失败"));
        }
    }

    /**
     * 保存文件到本地
     *
     * @param fileName fileName
     * @param fileData fileData
     */
    private void saveFileToLocal(String fileName, byte[] fileData) {
        String workingDir = System.getProperty("user.dir");
        String uploadDir = workingDir + File.separator + "web" + File.separator + "oss";

        // 创建目录（如果不存在）
        FileUtil.mkdir(uploadDir);

        String filePath = uploadDir + File.separator + fileName;
        File file = new File(filePath);
        if (!file.exists()) {
            try (FileOutputStream fos = new FileOutputStream(file)) {
                IoUtil.write(fos, true, fileData);
                log.info("File saved: " + filePath);
            } catch (Exception e) {
                log.error("File upload Exception: " + filePath, e);
            }
        } else {
            log.debug("File already exists, skipping: " + filePath);
        }
    }

}
